package com.zanxiang.game.module.manage.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.back.base.pojo.enums.OrderStatusEnum;
import com.zanxiang.game.module.manage.constant.RedisKeyConstant;
import com.zanxiang.game.module.manage.enums.ExpireTimeEnum;
import com.zanxiang.game.module.manage.enums.KfRoomMsgOwnerEnum;
import com.zanxiang.game.module.manage.enums.KfRoomMsgTypeEnum;
import com.zanxiang.game.module.manage.enums.KfWebSocketMsgEnum;
import com.zanxiang.game.module.manage.pojo.dto.KfAppletMsgDTO;
import com.zanxiang.game.module.manage.pojo.dto.KfWebSocketMsgDTO;
import com.zanxiang.game.module.manage.pojo.dto.PayApplicationDTO;
import com.zanxiang.game.module.manage.service.*;
import com.zanxiang.game.module.manage.service.api.KfWxApiService;
import com.zanxiang.game.module.manage.utils.FileUtil;
import com.zanxiang.game.module.manage.utils.RedisUtil;
import com.zanxiang.game.module.mybatis.entity.*;
import com.zanxiang.module.oss.service.IOssService;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.bean.BeanUtil;
import com.zanxiang.module.util.exception.BaseException;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.util.UriComponentsBuilder;

import java.io.IOException;
import java.math.BigDecimal;
import java.net.URI;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;

/**
 * @author : lingfeng
 * @time : 2024-02-28
 * @description : 小程序客服消息处理
 */
@Slf4j
@Service
public class KfAppletMsgServiceImpl implements IKfAppletMsgService {

    @Value("${media.realm-name}")
    private String realmName;

    @Value("${payConfig.wxPay.customH5Url}")
    private String customH5Url;

    @Autowired
    private KfWxApiService kfWxApiService;

    @Autowired
    private IOrderService orderService;

    @Autowired
    private IGamePayWayService gamePayWayService;

    @Autowired
    private IPayApplicationService payApplicationService;

    @Autowired
    private IOssService ossService;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private IKfSessionUserService kfSessionUserService;

    @Autowired
    private IKfRoomService kfRoomService;

    @Autowired
    private IGameAppletService gameAppletService;

    @Autowired
    private IUserService userService;

    @Autowired
    private IGameUserRoleService gameUserRoleService;

    @Autowired
    private IKfRoomMsgService kfRoomMsgService;

    @Autowired
    private RedisUtil<String> redisUtil;

    @Override
    public void appletMsg(String postData) {
        log.error("接收到SDK转发的小程序消息, postData : {}", postData);
        KfAppletMsgDTO kfAppletMsgDTO = JsonUtil.toObj(postData, KfAppletMsgDTO.class);
        GameApplet gameApplet = gameAppletService.getOne(new LambdaQueryWrapper<GameApplet>()
                .eq(GameApplet::getGhId, kfAppletMsgDTO.getToUserName()));
        //小游戏信息不存在, 消息不处理
        if (gameApplet == null) {
            return;
        }
        //事件消息
        if (Objects.equals(kfAppletMsgDTO.getMsgType(), KfAppletMsgDTO.MSG_TYPE_EVENT)) {
            this.eventMsgHandle(gameApplet, kfAppletMsgDTO);
            return;
        }
        //非玩家消息, 不做处理
        if (kfAppletMsgDTO.getMsgId() == null) {
            return;
        }
        //查询玩家的房间连接状态
        KfRoom kfRoom = kfRoomService.getOne(new LambdaQueryWrapper<KfRoom>()
                .eq(KfRoom::getOpenId, kfAppletMsgDTO.getFromUserName())
                .eq(KfRoom::getGameId, gameApplet.getGameId())
                .eq(KfRoom::getOnline, Boolean.TRUE));
        //构造房间消息
        KfWebSocketMsgDTO.MsgContentBean msgContent = this.getMsgContent(kfAppletMsgDTO);
        KfRoomMsg kfRoomMsg = this.transform(kfAppletMsgDTO, gameApplet, kfRoom, postData, msgContent);
        //判断是否请求支付链接
        String orderId = redisUtil.getCache(RedisKeyConstant.GAME_CUSTOM_PAY + kfAppletMsgDTO.getFromUserName());
        if (Strings.isNotBlank(orderId) && Objects.equals(kfAppletMsgDTO.getContent(), KfAppletMsgDTO.MSG_CONTENT_PAY)) {
            Order order = orderService.getOne(new LambdaQueryWrapper<Order>()
                    .eq(Order::getOrderId, orderId)
                    .and(qw -> qw.eq(Order::getStatus, OrderStatusEnum.READY_PAY.getValue())
                            .or().eq(Order::getStatus, OrderStatusEnum.WAIT_PAY.getValue())
                    )
            );
            if (order != null) {
                this.sendCustomPayMessage(gameApplet.getGameId(), kfAppletMsgDTO.getFromUserName(), kfRoom, order);
                kfRoomMsg.setReadStatus(Boolean.TRUE);
                kfRoomMsgService.save(kfRoomMsg);
                return;
            }
        }
        //保存房间消息
        kfRoomMsgService.save(kfRoomMsg);
        //玩家状态更新为待接入状态
        if (kfRoom == null) {
            kfSessionUserService.update(new LambdaUpdateWrapper<KfSessionUser>()
                    .set(KfSessionUser::getIsWait, Boolean.TRUE)
                    .set(KfSessionUser::getWaitStartTime, LocalDateTime.now())
                    .set(KfSessionUser::getUpdateTime, LocalDateTime.now())
                    .eq(KfSessionUser::getOpenId, kfAppletMsgDTO.getFromUserName()));
        }
        //消息转发到redis频道
        this.pushMessage(this.transform(kfRoom, gameApplet.getGameId(), kfRoomMsg, msgContent));
    }

    private void eventMsgHandle(GameApplet gameApplet, KfAppletMsgDTO kfAppletMsgDTO) {
        //事件类型
        String event = kfAppletMsgDTO.getEvent();
        //非玩家打开会话消息, 不处理
        if (!Objects.equals(event, KfAppletMsgDTO.EVENT_USER_ENTER_TEMP_SESSION)) {
            return;
        }
        //保存玩家信息
        this.kfSessionUserUpdateSave(kfAppletMsgDTO, gameApplet);
        //判断玩家是否存在客服支付订单
        String customPaySign = RedisKeyConstant.GAME_CUSTOM_PAY_SIGN + kfAppletMsgDTO.getFromUserName();
        String orderId = redisUtil.getCache(customPaySign);
        if (Strings.isBlank(orderId)) {
            return;
        }
        //判断玩家当天是否发送过消息
        if (kfRoomMsgService.count(new LambdaQueryWrapper<KfRoomMsg>()
                .eq(KfRoomMsg::getOpenId, kfAppletMsgDTO.getFromUserName())
                .ge(KfRoomMsg::getCreateTime, LocalDateTime.now().with(LocalTime.MIDNIGHT))
        ) <= 0) {
            return;
        }
        //查询订单
        Order order = orderService.getOne(new LambdaQueryWrapper<Order>()
                .eq(Order::getOrderId, orderId)
                .and(qw -> qw.eq(Order::getStatus, OrderStatusEnum.READY_PAY.getValue())
                        .or().eq(Order::getStatus, OrderStatusEnum.WAIT_PAY.getValue())
                )
        );
        if (order == null) {
            return;
        }
        this.sendCustomPayMessage(gameApplet.getGameId(), kfAppletMsgDTO.getFromUserName(), null, order);
        redisUtil.deleteCache(customPaySign);
        redisUtil.setCache(RedisKeyConstant.GAME_CUSTOM_PAY + kfAppletMsgDTO.getFromUserName(),
                orderId, ExpireTimeEnum.FIVE_MIN.getTime());
    }

    private KfWebSocketMsgDTO transform(KfRoom kfRoom, Long gameId, KfRoomMsg kfRoomMsg, KfWebSocketMsgDTO.MsgContentBean msgContent) {
        //消息类型
        KfWebSocketMsgEnum kfWebSocketMsgEnum = kfRoom == null ? KfWebSocketMsgEnum.WEBSOCKET_MSG_WAIT_LIST
                : KfWebSocketMsgEnum.WEBSOCKET_MSG_ROOM_MSG;
        KfWebSocketMsgDTO kfWebSocketMsgDTO = KfWebSocketMsgDTO.builder()
                .webSocketMsgType(kfWebSocketMsgEnum)
                .kfUserId(kfRoom == null ? null : kfRoom.getKfUserId())
                .gameId(gameId)
                .roomId(kfRoom == null ? null : kfRoom.getId())
                .build();
        //待接入消息
        if (Objects.equals(kfWebSocketMsgEnum, KfWebSocketMsgEnum.WEBSOCKET_MSG_WAIT_LIST)) {
            List<KfWebSocketMsgDTO.WaitUserBean> waitUserList = kfSessionUserService.getWaitUserList(gameId);
            kfWebSocketMsgDTO.setWaitUserList(waitUserList);
        }
        //房间消息
        if (Objects.equals(kfWebSocketMsgEnum, KfWebSocketMsgEnum.WEBSOCKET_MSG_ROOM_MSG)) {
            //消息对象
            KfWebSocketMsgDTO.RoomMsgBean roomMsgBean = BeanUtil.copy(kfRoomMsg, KfWebSocketMsgDTO.RoomMsgBean.class);
            roomMsgBean.setContent(msgContent);
            kfWebSocketMsgDTO.setRoomMsgList(Collections.singletonList(roomMsgBean));
            //设置房间列表和游戏列表
            if (kfRoom != null) {
                kfWebSocketMsgDTO.setGameList(kfRoomService.getKfGameByGameId(gameId, kfRoom.getKfUserId()));
                kfWebSocketMsgDTO.setRoomList(kfRoomService.getRoomByRoomId(kfRoom.getId()));
            }
        }
        return kfWebSocketMsgDTO;
    }

    private void kfSessionUserUpdateSave(KfAppletMsgDTO kfAppletMsgDTO, GameApplet gameApplet) {
        KfSessionUser kfSessionUser = kfSessionUserService.getById(kfAppletMsgDTO.getFromUserName());
        //查询用户
        User user = userService.getOne(new LambdaQueryWrapper<User>().eq(User::getOpenId, kfAppletMsgDTO.getFromUserName()));
        //最近角色信息
        GameUserRole gameUserRole = null;
        if (user != null) {
            gameUserRole = gameUserRoleService.getLastGameUserRoleName(user.getId(), user.getGameId());
        }
        //不存在玩家信息, 创建保存
        if (kfSessionUser == null) {
            kfSessionUserService.save(this.transform(kfAppletMsgDTO, gameApplet, user, gameUserRole));
            return;
        }
        //存在, 更新玩家信息
        kfSessionUserService.update(new LambdaUpdateWrapper<KfSessionUser>()
                .set(KfSessionUser::getUserId, user == null ? null : user.getId())
                .set(KfSessionUser::getLastRoleId, gameUserRole == null ? null : gameUserRole.getRoleId())
                .set(KfSessionUser::getLastRoleName, gameUserRole == null ? "神秘人[未创角]" : gameUserRole.getRoleName())
                .set(gameUserRole != null, KfSessionUser::getServerId, gameUserRole == null ? null : gameUserRole.getServerId())
                .set(gameUserRole != null, KfSessionUser::getServerName, gameUserRole == null ? null : gameUserRole.getServerName())
                .set(KfSessionUser::getUpdateTime, LocalDateTime.now())
                .eq(KfSessionUser::getOpenId, kfAppletMsgDTO.getFromUserName()));
    }

    private KfSessionUser transform(KfAppletMsgDTO kfAppletMsgDTO, GameApplet gameApplet, User user, GameUserRole gameUserRole) {
        return KfSessionUser.builder()
                .openId(kfAppletMsgDTO.getFromUserName())
                .gameId(gameApplet.getGameId())
                .userId(user == null ? null : user.getId())
                .isWait(Boolean.FALSE)
                .lastRoleId(gameUserRole == null ? null : gameUserRole.getRoleId())
                .lastRoleName(gameUserRole == null ? "神秘人[未创角]" : gameUserRole.getRoleName())
                .serverId(gameUserRole == null ? null : gameUserRole.getServerId())
                .serverName(gameUserRole == null ? null : gameUserRole.getServerName())
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .build();
    }

    private KfRoomMsg transform(KfAppletMsgDTO kfAppletMsgDTO, GameApplet gameApplet, KfRoom kfRoom, String postData,
                                KfWebSocketMsgDTO.MsgContentBean msgContent) {
        return KfRoomMsg.builder()
                .msgId(String.valueOf(kfAppletMsgDTO.getMsgId()))
                .msgType(kfAppletMsgDTO.getMsgType())
                .gameId(gameApplet.getGameId())
                .openId(kfAppletMsgDTO.getFromUserName())
                .userId(kfRoom == null ? null : kfRoom.getUserId())
                .readStatus(Boolean.FALSE)
                .roomId(kfRoom == null ? null : kfRoom.getId())
                .kfUserId(kfRoom == null ? null : kfRoom.getKfUserId())
                .msgOwner(KfRoomMsgOwnerEnum.KF_MSG_OWNER_USER.getValue())
                .content(JsonUtil.toString(msgContent))
                .source(postData)
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .build();
    }

    private KfWebSocketMsgDTO.MsgContentBean getMsgContent(KfAppletMsgDTO kfAppletMsgDTO) {
        KfWebSocketMsgDTO.MsgContentBean msgContentBean = new KfWebSocketMsgDTO.MsgContentBean();
        if (Objects.equals(KfRoomMsgTypeEnum.KF_MSG_TYPE_TEXT.getValue(), kfAppletMsgDTO.getMsgType())) {
            msgContentBean.setText(kfAppletMsgDTO.getContent());
        }
        if (Objects.equals(KfRoomMsgTypeEnum.KF_MSG_TYPE_IMAGE.getValue(), kfAppletMsgDTO.getMsgType())) {
            msgContentBean.setImage(this.mediaConvertOss(kfAppletMsgDTO.getPicUrl()));
        }
        return msgContentBean;
    }

    private String mediaConvertOss(String mediaUrl) {
        //资源转换
        MultipartFile multipartFile = FileUtil.urlToMultipartFile(mediaUrl);
        //生成唯一文件名
        String fileName = Long.toString(System.currentTimeMillis(), 36) + SecurityUtil.getUserId() + ".jpg";
        try {
            ossService.upload("image/" + fileName, multipartFile.getInputStream());
        } catch (IOException e) {
            log.error("文件上传oss异常, mediaUrl : {}, e : {}", mediaUrl, e.getMessage());
            throw new BaseException("文件上传oss异常");
        }
        //oss资源地址
        return this.realmName + "image/" + fileName;
    }

    private void sendCustomPayMessage(Long gameId, String openId, KfRoom kfRoom, Order order) {
        //查询订单支付方式
        GamePayWay gamePayWay = gamePayWayService.getById(order.getGamePayWayId());
        //查询支付应用信息
        PayApplicationDTO payApplicationDTO = payApplicationService.getPayApplicationByPayBoxId(gamePayWay.getPayBoxId());
        //客服支付链接显示图片地址
        String thumbUrl = gamePayWay.getThumbUrl();
        //支付配置参数判断
        if (Strings.isBlank(thumbUrl)) {
            log.error("客服消息卡片图片地址不存在, gamePayWayDTO : {}", JsonUtil.toString(gamePayWay));
            return;
        }
        //订单金额
        BigDecimal amount = order.getAmount();
        //构造跳转链接url
        URI url = UriComponentsBuilder.fromHttpUrl(this.customH5Url)
                .queryParam("appId", payApplicationDTO.getAppId())
                .queryParam("orderId", order.getOrderId())
                .queryParam("amount", amount)
                .queryParam("description", "购买" + amount + "元档充值")
                .build().toUri();
        //link参数构造
        Map<String, Object> linkMap = new HashMap<>(4);
        linkMap.put("title", "点我充值");
        linkMap.put("description", "点我充值" + amount + "元,用于购买" + amount + "元档充值");
        linkMap.put("url", url);
        linkMap.put("thumb_url", thumbUrl);
        //发送消息
        Map<String, Object> msgParamMap = new HashMap<>(3);
        msgParamMap.put("touser", openId);
        msgParamMap.put("msgtype", KfRoomMsgTypeEnum.KF_MSG_TYPE_LINK.getValue());
        msgParamMap.put(KfRoomMsgTypeEnum.KF_MSG_TYPE_LINK.getValue(), linkMap);
        kfWxApiService.sendCustomMessageApi(gameId, msgParamMap);
        //返回发送的消息内容
        log.error("客服支付, 发送支付信息 : {}", JsonUtil.toString(msgParamMap));
        kfRoomMsgService.save(this.transform(openId, gameId, kfRoom, JsonUtil.toString(msgParamMap)));
    }

    private KfRoomMsg transform(String openId, Long gameId, KfRoom kfRoom, String msgContent) {
        return KfRoomMsg.builder()
                .msgId(UUID.randomUUID().toString().replace("-", ""))
                .msgType(KfRoomMsgTypeEnum.KF_MSG_TYPE_LINK.getValue())
                .gameId(gameId)
                .openId(openId)
                .userId(kfRoom == null ? null : kfRoom.getUserId())
                .readStatus(kfRoom != null)
                .roomId(kfRoom == null ? null : kfRoom.getId())
                .kfUserId(kfRoom == null ? null : kfRoom.getKfUserId())
                .msgOwner(KfRoomMsgOwnerEnum.KF_MSG_OWNER_SYSTEM.getValue())
                .content(msgContent)
                .source(msgContent)
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .build();
    }

    /**
     * 消息发送到redis广播
     */
    private void pushMessage(KfWebSocketMsgDTO kfWebSocketMsgDTO) {
        redisTemplate.convertAndSend(RedisKeyConstant.KF_MSG_REDIS_LISTEN_TOPIC, JsonUtil.toString(kfWebSocketMsgDTO));
    }
}
